/*
 * Copyright (c) 2009 Lukas Mejdrech
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup netif
 *  @{
 */

/** @file
 *  Network interface module skeleton.
 */

#ifndef __NET_NETIF_H__
#define __NET_NETIF_H__

#include <rwlock.h>

#include "../include/device.h"

/** Sends the notification message to the registered network interface layer module.
 *  Wrapper of the async_msg_3().
 *  The first message argument is always the device identifier.
 *  @param device The device specific data. Input parameter.
 *  @param message The message to be sent. Input parameter.
 *  @param arg2 The second message argument. Input parameter.
 *  @param arg3 The third message argument. Input parameter.
 */
#define nil_message( device, message, arg2, arg3 )	\
	if(( device )->nil_phone >= 0 ) async_msg_3(( device )->nil_phone, ( message ), ( device )->device_id, arg2, arg3 )

/** Network interface module skeleton global data.
 */
typedef struct netif_globals	netif_globals_t;

/** Type definition of the device specific data.
 *  @see netif_device
 */
typedef struct netif_device	device_t;

/** Type definition of the device specific data pointer.
 *  @see netif_device
 */
typedef device_t *			device_ref;

/** Device map.
 *  Maps device identifiers to the network interface device specific data.
 *  @see device.h
 */
DEVICE_MAP_DECLARE( device_map, device_t );

/** Network interface device specific data.
 */
struct	netif_device{
	/** Device identifier.
	 */
	device_id_t	device_id;
	/** Receiving network interface layer phone.
	 */
	int		nil_phone;
	/** Actual device state.
	 */
	device_state_t	state;
	/** Driver specific data.
	 */
	void *		specific;
};

/** Network interface module skeleton global data.
 */
struct	netif_globals{
	/** Networking module phone.
	 */
	int		networking_phone;
	/**	Device map.
	 */
	device_map_t	device_map;
	/** Safety lock.
	 */
	rwlock_t	lock;
};

/**	Finds the device specific data.
 *  @param device_id The device identifier. Input parameter.
 *  @param device The device specific data. Output parameter.
 *  @returns EOK on success.
 *  @returns ENOENT if device is not found.
 *  @returns EPERM if the device is not initialized.
 */
int	find_device( device_id_t device_id, device_ref * device );

/** Clears the usage statistics.
 *  @param stats The usage statistics. Input parameter.
 */
void	null_device_stats( device_stats_ref stats );

// prepared for future optimalizations
/** \todo
 */
void	netif_pq_release( packet_id_t packet );

/** \todo
 */
packet_t netif_packet_get_1( size_t content );

#endif

/** @}
 */

